<?php

namespace Drupal\media_pdf_thumbnail\Plugin\QueueWorker;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImageManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Generates a thumbnail for media (PDF) having enabled the support.
 *
 * @QueueWorker(
 *   id = "media_pdf_thumbnail_generate_thumbnail",
 *   title = @Translation("Generates thumbnail for Media (PDF)"),
 *   cron = {"time" = 5}
 * )
 */
class GenerateThumbnailWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The thumbnail image manager.
   *
   * @var \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImageManager
   */
  protected $thumbnailImageManager;

  /**
   * Constructs a new GenerateThumbnailWorker object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImageManager $thumbnail_image_manager
   *   The thumbnail image manager.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, MediaPdfThumbnailImageManager $thumbnail_image_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->thumbnailImageManager = $thumbnail_image_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('media_pdf_thumbnail.image.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $use_queue = (bool) $this->configFactory->get('media_pdf_thumbnail.bundles.settings')->get('use_queue');
    if (!$use_queue) {
      return;
    }

    /** @var \Drupal\media\Entity\Media $media */
    $media = $this->entityTypeManager->getStorage('media')->load($data['id']);
    // Checks wheiter the media thumbnail still needs update to avoid duplicate
    // thumbnail generation.
    $should_create = $media && _media_pdf_thumbnail_should_update_thumbnail($media);
    if ($should_create) {
      // Generates the media thumbnail.
      $this->thumbnailImageManager->createThumbnail($media);
      // Saves the new thumbnail.
      $media->save();
    }
  }

}
